#!/usr/bin/env /opt/sfw/bin/python

# Copyright (c) 2004 Benjamin Jencks <ben@bjencks.net>
# Licensed under the GNU GPL For full terms see the file COPYING.
# Originally inspired by wicap-php by Caleb Phillips. PHP code
# is his with modifications.

# Logging output:
# C ip mac
#  Canceled lease
# D ip mac
#  Lease deleted because mac is gone
# N ip mac
#  New lease created because it showed up in arp (not authed)
# A ip mac table
#  Client added to table
# L message
#  Status message


import time, os, os.path, re, socket, threading
import signal, sys, select
import datetime
from DateFormat import DateFormat

# location of pfctl
# CHANGE TO PREFERRED LOCATION
PFCTL="/opt/sfw/bin/sudo /home/....../ipfilter.sh %(op)s %(ip)s"
#PFCTL="echo %(op)s %(ip)s"

ARP="/usr/sbin/arp"

FIFOPATH="/var/captive/ipc"

#table to add allowed clients to
ALLOWTABLE="wi_allow"

# function to decide whether to pay attention to
# an address
netre =  re.compile("172.19.([0-9.]+)")
NET_MATCH=netre.match

# how long an auth is valid for
TIMEOUT=24*60*60

POLLINTERVAL=1

# ugly handling in case of signal during IO call
def doIO(func):
    while True:
        try:
            ret = func()
            return ret
        except IOError, val:
            if val[0] == 4: # 4 == EINTR
                continue
            else:
                raise sys.exc_type, sys.exc_value
class Lease(object):
    """Represents a lease, with an ip and a mac.

    Passing table and authTime are supported for reading from a file.

    Create the lease when it shows up in the arp table, then call auth()
    when it auths. Or, if it's a whitelisted client, it will be allowed
    as soon as the lease is created.
    """
    def __init__(self, ip, mac):
        log("N %s %s" % (ip, mac))
        self.__timer = threading.Timer(TIMEOUT, self.cancel)
        self.__ip = ip
        self.__mac = mac
        self.__authed = False
    def getIP(self):
        return self.__ip
    def getMac(self):
        return self.__mac

    def auth(self):
        if not self.__authed:
            self.__timer.start()
            log("A %s %s" % (self.getIP(), self.getMac()))
            os.system(PFCTL%{"op":"add", "ip":self.getIP()})
            self.__authed = True
        else:
            self.__timer.cancel()
            self.__timer = threading.Timer(TIMEOUT, self.cancel)
            self.__timer.start()
            log("R %s %s" % (self.getIP(), self.getMac()))
    
    def cancel(self):
        self.__timer.cancel()
        log("C %s %s" % (self.getIP(), self.getMac()))
        os.system(PFCTL%{"op":"del", "ip":self.getIP()})
        self.__timer = threading.Timer(TIMEOUT, self.cancel)
        self.__authed = False
        

class LeaseList(object):
    def __init__(self):
        self.__leases = []
#        self.__macre = re.compile(".* \(([0-9.]+)\) at ([0-9a-f:]+|\(incomplete\)) on .*")
        self.__macre = re.compile(".* ([0-9.]+) .* [0-9.]+.* ([0-9a-f:]+|U)")


    def refresh(self):
        arp = os.popen(ARP + " -na | tail +5")
        arpLines = doIO(arp.readlines)
        arp.close()
        map = {}
        # this loop creates an ip->mac mapping
        # that contains IPs that NET_MATCH returns true
        for line in arpLines:
            match = self.__macre.match(line)
            if match:
                ip, mac = match.group(1), match.group(2)
                if NET_MATCH(ip) and mac != 'U':
                    map[ip] = mac
            else:
                log("L Bad arp: " + line)
        # find dead leases, and filter map for live leases
        for lease in self.__leases[:]:
            if lease.getIP() in map and lease.getMac() == map[lease.getIP()]:
                del map[lease.getIP()]
            else:
                lease.cancel()
                log("D " + lease.getIP() + " " + lease.getMac())
                self.__leases.remove(lease)
                os.system("cat /dev/null > /var/captive/fifo")
        # map now only contains ip->macs that don't have leases
        # find new leases
        for ip, mac in map.items():
            self.__leases.append(Lease(ip, mac))

    def auth(self, ip):
        for lease in self.__leases:
            if lease.getIP() == ip:
                lease.auth()
                return

def log(str):
    d = datetime.datetime.now()
    df = DateFormat(d)
    print df.format('D j H:i:s ') + str
    sys.stdout.flush()


def runServer():
    log("L Starting server")

    os.system("rm " + FIFOPATH)
    fifo = socket.socket ( socket.AF_UNIX, socket.SOCK_DGRAM )
    fifo.bind (FIFOPATH)
    os.system("chmod 770 " + FIFOPATH)

    while True:

         data,addr = fifo.recvfrom(1024)
         ip = data.rstrip()

         try:
             socket.inet_aton(ip)
             leaselist.auth(ip)
         except socket.error:
             pass

class Poller(object):
    def __init__(self):
        self.__running = True

    def pollLoop(self):
        log("L Starting polling")
        while self.__running:
            leaselist.refresh()
            time.sleep(POLLINTERVAL)
        log("L Ending polling")
    def stop(self):
        self.__running = False
        
def spawn():
    thr = threading.Thread(target=runServer)
    thr.setDaemon(True)
    thr.start()

def quit(signum, frame, poller):
    poller.stop()
    log("L Shutting down")

os.system("echo init")

poller = Poller()
handler = lambda signum, frame: quit(signum, frame, poller)
signal.signal(signal.SIGHUP, handler)
signal.signal(signal.SIGTERM, handler)
signal.signal(signal.SIGINT, handler)

leaselist = LeaseList()
spawn()
poller.pollLoop()
