#!/bin/sh

# config section

IPF_SBIN_PREFIX=/usr/sbin
IPF=${IPF_SBIN_PREFIX}/ipf
IPNAT=${IPF_SBIN_PREFIX}/ipnat
IPPOOL=${IPF_SBIN_PREFIX}/ippool
EXTERNALINT="bge0"
INTERNALINT="bge179002"

RULEFILENAT=/var/captive/rules-nat

PREFIX=/home/..../captive-0.1
IPFFILEINIT=${PREFIX}/ipf.conf

# end of config section

umask 077

initrules(){

   cat /dev/null > $RULEFILENAT

   COUNT=2

   cat << END-OF-FILE >> $RULEFILENAT
map $EXTERNALINT 172.18.0.0/16 -> 147.83.20.245/32 portmap tcp/udp 20000:30000
map $EXTERNALINT 172.18.0.0/16 -> 147.83.20.245/32
map $EXTERNALINT 172.19.0.0/16 -> 147.83.20.246/32 portmap tcp/udp 30000:40000
map $EXTERNALINT 172.19.0.0/16 -> 147.83.20.246/32
END-OF-FILE

   while [ $COUNT -ne "255" ]; do
      echo "rdr $INTERNALINT from 172.19.1.$COUNT/32 to any port = 80 -> 172.19.0.1 port 81" >> $RULEFILENAT
      COUNT=`expr $COUNT + 1`
   done

   $IPF -A -F a -f $IPFFILEINIT > /dev/null 2>&1
   $IPNAT -C -f $RULEFILENAT > /dev/null 2>&1

}

addaddress(){

   # add address to allowed pool

   $IPPOOL -a -m 101 -i $1/32 > /dev/null 2>&1

   # remove redirect

   cat $RULEFILENAT | grep -v $1 > $RULEFILENAT.new
   mv $RULEFILENAT.new $RULEFILENAT
   sort -u -o $RULEFILENAT $RULEFILENAT 
   cat $RULEFILENAT | $IPNAT -C -f - > /dev/null 2>&1

}

deladdress(){

   # remove address to allowed pool

   $IPPOOL -r -m 101 -i $1/32 > /dev/null 2>&1

   # add redirect

   echo "rdr $INTERNALINT from $1/32 to any port = 80 -> 172.19.0.1 port 81" >> $RULEFILENAT
   sort -u -o $RULEFILENAT $RULEFILENAT
   cat $RULEFILENAT | $IPNAT -C -f - > /dev/null 2>&1

}

if [ $# -ge 3 ]; then echo USE: `basename $0` "<add|del> <ip> | <init>"; exit 1; fi

COMMAND="$1"
IP="$2"

case $COMMAND in
add) addaddress $IP;;
del) deladdress $IP;;
init) initrules
      exit 0;;
*) echo USE: `basename $0` "<add|del> <ip> | <init>"; exit 1;;
esac

